import sys
import os


def stack():
    return_str = ""
    f = sys._getframe()
    f = f.f_back  # first frame is detailtrace, ignore it
    while hasattr(f, "f_code"):
        code = f.f_code
        if return_str == "":
            return_str = "%s(%s:%s)" % (os.path.basename(code.co_filename),
                                        code.co_name,
                                        f.f_lineno)
        else:
            return_str = "%s(%s:%s)->" % (os.path.basename(code.co_filename),
                                          code.co_name,
                                          f.f_lineno) + return_str

        f = f.f_back
    print(return_str)
    return return_str




# 定义Exception

class ConstError(TypeError):
    """
    常量类型检查错误
    """
    pass

class BeeError(Exception):
    pass


class CodedError(BeeError):

    def __init__(self, code: int = 0, message: str= None, detail: str = None): # real signature unknown
        self.code = code
        self.message = message
        self.detail = detail

    def __str__(self):
        if self.detail == None or self.detail == "":
            return "{}({})".format(self.message, self.code)

        return "{}({}): {}".format(self.message, self.code, self.detail)






from typing import Dict

from typing_extensions import KT, VT


class Map(Dict[KT, VT]):

    def __init__(self, names=(), values=(), **kw):
        super(Map, self).__init__(**kw)
        for k, v in zip(names, values):
            self[k] = v

    def __getattr__(self, key: str):
        self._check_key_type(key)
        try:
            return self[key]
        except KeyError:
            raise AttributeError(r"'Map' object has no attribute '%s'" % key)

    def __setattr__(self, key: str, value):
        self._check_key_type(key)
        self[key] = value

    def __delattr__(self, item):
        del self[item]

    def set(self, key, value):
        self._check_key_type(key)
        self[key] = value

    def get(self, key: str) -> VT:
        self._check_key_type(key)
        if self.contains(key):
            return self[key]
        else:
            return None

    def find(self, key: str):
        if self.contains(key):
            return self.get(key)

        i = key.find(".")
        while i != -1:
            v = self.get(key[:i])
            if v != None:
                tmp, ok = self.try_convert(v)
                # print("k=%s, v=%s, k2=%s, ok=%s" % (key[:i], str(tmp), key[i+1:], str(ok)))
                if ok:
                    return tmp.find(key[i+1:])
            else:
                return None
        return None


    def _check_key_type(self, key: str):
        if isinstance(key, str) == False:
            raise TypeError("args'type %s is not str" % str(type(key)))

    def remove(self, key):
        self._check_key_type(key)
        delattr(self, key)

    def keys(self):
        return list(map(lambda x: x[0], self.items()))

    def values(self):
        return list(map(lambda x: x[1], self.items()))

    def contains(self, key: str):
        self._check_key_type(key)
        return self.keys().__contains__(key)

    def merge(self, src: "Map"):
        for k, v in src.items():
            if self.contains(k):
                tv = self.get(k)
                m1, ok1 = self.try_convert(tv)
                m2, ok2 = self.try_convert(v)
                if ok1 and ok2:
                    m1.merge(m2)
            else:
                tm, ok = self.try_convert(v)
                if ok:
                    self.set(k, tm)
                else:
                    self.set(k, v)

    def cover(self, src: "Map"):
        for k, v in src.items():
            if self.contains(k):
                tv = self.get(k)
                self.set(k, v)
                if tv == None:
                    self.set(k, v)
                else:
                    m1, ok1 = self.try_convert(tv)
                    m2, ok2 = self.try_convert(v)
                    if ok1 and ok2:
                        m1.cover(m2)
            else:
                tm, ok = self.try_convert(v)
                if ok:
                    self.set(k, tm)
                else:
                    self.set(k, v)


    def try_convert(self, i) -> ("Map", bool):
        t = type(i)
        if t == type(self):
            return i, True
        elif t == dict:
            return Map.from_dict(i), True

        return None, False


    def empty(self):
        for x in self.keys():
            delattr(self, x)

    @staticmethod
    def from_dict(d={}):
        map = Map()
        if d != None:
            for k, v in d.items():
                if isinstance(v, dict):
                    map.set(str(k), Map.from_dict(v))
                else:
                    map.set(str(k), v)
        return map

    def __missing__(self, key):
        value = self[key] = type(self)()
        return value



from typing import List

from bee_util.data.map import Map


class Option(Map):

    def __init__(self, name: str, value: str):
        self.name = name
        self.value = value

def parse_option(s: str, sep: str) -> Option:
    pair = s.split(sep)
    return Option(name=pair[0], value=pair[1])



class Options(List[Option]):
    pass

def parse_options(s: str, sep1: str, sep2: str) -> Options:
    parts = s.split(sep1)
    opts = []
    for v in parts:
        opts.append(parse_option(v, sep2))
    return opts




# Ported from https://github.com/graham/python_xid which almost a direct copy of xid(https://github.com/rs/xid)


from bee_util.data.guid import base32hex

import hashlib
import platform
import os
import time
import datetime
import threading

from bee_util.errors.error import BeeError


class InvalidGuidError(BeeError):
    pass

trimLen = 20
encodedLen = 24
decodedLen = 14
rawLen = 12


def read_int():
    # type: () -> int
    buf = str(os.urandom(3))
    buford = list(map(ord, buf))
    return buford[0] << 16 | buford[1] << 8 | buford[2]


def read_machine_id():
    """
    read machine's id
    """
    try:
        hostname = platform.node()
        hw = hashlib.md5()
        hw.update(hostname.encode('utf-8'))
        # return md5.hexdigest()
        val = str(hw.digest()[:3])
        return list(map(ord, val))
    except:
        buf = os.urandom(3)
        return list(map(ord, str(buf)))


pid = os.getpid()
machineID = read_machine_id()
lock = threading.Lock()


def generateNextId():
    id = read_int()

    while True:
        new_id = id + 1
        id += 1
        yield new_id


objectIDGenerator = generateNextId()


def generate_new_xid():
    # type: () -> List[int]
    now = int(time.time())
    id = [0] * rawLen

    id[0] = (now >> 24) & 0xff
    id[1] = (now >> 16) & 0xff
    id[2] = (now >> 8) & 0xff
    id[3] = (now) & 0xff

    id[4] = machineID[0]
    id[5] = machineID[1]
    id[6] = machineID[2]

    id[7] = (pid >> 8) & 0xff
    id[8] = (pid) & 0xff

    lock.acquire()
    i = next(objectIDGenerator)
    lock.release()

    id[9] = (i >> 16) & 0xff
    id[10] = (i >> 8) & 0xff
    id[11] = (i) & 0xff

    return id


class Guid(object):
    def __init__(self, id=None):
        # type: (List[int]) -> None
        if id is None:
            id = generate_new_xid()
        self.value = id

    def pid(self):
        # type: () -> int
        return (self.value[7] << 8 | self.value[8])

    def counter(self):
        # type: () -> int
        return (self.value[9] << 16 |
                self.value[10] << 8 |
                self.value[11])

    def machine(self):
        # type: () -> str
        return ''.join(map(chr, self.value[4:7]))

    def datetime(self):
        return datetime.datetime.fromtimestamp(self.time())

    def time(self):
        # type: () -> int
        return (self.value[0] << 24 |
                self.value[1] << 16 |
                self.value[2] << 8 |
                self.value[3])

    def string(self):
        # type: () -> str
        byte_value = self.bytes()
        return base32hex.b32encode(byte_value).lower()[:trimLen]

    def bytes(self):
        # type: () -> str
        return ''.join(map(chr, self.value))

    def __repr__(self):
        return "<Guid '%s'>" % self.__str__()

    def __str__(self):
        return self.string()

    def __lt__(self, arg):
        # type: (Guid) -> bool
        return self.string() < arg.string()

    def __gt__(self, arg):
        # type: (Guid) -> bool
        return self.string() > arg.string()

    @classmethod
    def from_string(cls, s):
        # type: (str) -> Guid
        val = base32hex.b32decode(s.upper())
        value_check = [0 <= x < 255 for x in val]

        if not all(value_check):
            raise InvalidGuidError(s)

        return cls(val)


